"""
Handles Treasury Quants API Call
- GET
- POST

GitHub Description:
    This is the connection class that contains the HTTP protocol as well as the running acount balance, etc..
"""

# TreasuryQuants.com Ltd.
# email: contact@treasuryquants.com
# Note: this software is provided "as-is" under the agreed terms of your account.
#       For more information see https://treasuryquants.com/terms-of-services/

import requests
import datetime
from . import TQResponse, TQRequests


class Message:
    def __init__(self, is_OK, content):
        """
        Initializes Message Class object.

        Args:
            is_OK: Type[bool], False <- Response Error | True <- Response Result.
            content: Type[dict].

        """
        self.is_OK = is_OK
        self.content = content


class Connection:

    def __init__(self, email,  url, minutes_to_expiry=1):
        """
        Initializes Connection Class object.

        Args:
            email: Type[str], provide something@domain.com (default <- TQConfig.email).
            is_post: Type[bool], provide True to use POST or False to use GET (default <- TQConfig.is_http_post).  
            url: Type[str], provide TQ api url (default <- TQConfig.url).
            minutes_to_expiry: Type[int], (default = 1).

        Attributes:
            email, is_post, url, minutes_to_expiry,
            token, expiry, cost, balance, client_id, source_id, 
        """
        self.response = TQResponse.ResponseClass()
        self.email = email
        self.url = url
        self.token = ""
        self.expiry = datetime.datetime.now()
        self.cost = 0
        self.balance = 0
        self.client_id = ""
        self.source_id = ""
        self.__minutes_to_expiry = minutes_to_expiry
        self.__function_name_account_token_create='account_token_create'# the function name for creating the token
        self.is_post = True

        result = requests.get(url=self.url)

        #
        # treat the redirect, if any.
        #
        result_url = result.url
        self_url = self.url
        if not result_url.endswith('/'):
            result_url += '/'
        if not self_url.endswith('/'):
            self_url += '/'
        self.url=result_url

    def send(self, request):
        """
        Uses [is_post] attribute of Connection Object to determine
        whether to do a GET request or POST request.

        Args:
            request: Type[Request], Note: Request is `TQ` class not `requests` package class.
                     Attribute:
                        params: Type[dict], have key `function_name`
                        needs_token: Type[bool]
        """
        if self.is_post:
            return self.post(request)
        return self.get(request)

    def post(self, request):
        """
        Pre-Check before sending POST request
        - checks if token is needed
            -- generate token, if previous token expired 
        """
        param_dictionary = request.params
        request_needs_token = request.needs_token
        if request_needs_token:
            # add token here
            now = datetime.datetime.now()
            if (self.expiry - now).total_seconds() / 60 < self.__minutes_to_expiry:
                request_account_token_create = TQRequests.request_account_token_create(self.email)
                message = self.__post(request_account_token_create.params)
                if not message.is_OK:
                    return message
                self.token = self.response.results[self.__function_name_account_token_create]
            param_dictionary['token'] = self.token
        return self.__post(param_dictionary)

    def get(self, request):
        """
        Pre-Check before sending GET request
        - checks if token is needed
            -- generate token, if previous token expired 
        """
        param_dictionary = request.params
        request_needs_token = request.needs_token
        if request_needs_token:
            # add token here
            now = datetime.datetime.now()
            if (self.expiry - now).total_seconds() / 60 < self.__minutes_to_expiry:
                request_account_token_create = TQRequests.request_account_token_create(self.email)
                message = self.__get(request_account_token_create.params)
                if not message.is_OK:
                    return message
                #self.token = list(message.content.values())[0]
                self.token = self.response.results[self.__function_name_account_token_create]
            param_dictionary['token'] = self.token
        return self.__get(param_dictionary)

    def __result_to_message(self, result):
        """
        Triggers parsing of GET/POST response &
        From parsing result, Updates Attribute (clint_id, source_id, balance, cost, expiry)

        Args:
            result: Type[requests HTTP Response Object]
        
        Returns:
            Type[TQ Message], Message Object
        """
        if result.text.lstrip()=='':
            return Message(False, {'internal error':"Response was empty."})
        store = TQResponse.StoreClass()
        #store.fromXml(result.text)
        store.fromJson(result.text)
        self.client_id = store.client_id
        self.source_id = store.source_id  # ip
        self.balance = store.response.balance
        self.cost = store.response.cost
        self.expiry = datetime.datetime.now() + datetime.timedelta(store.response.expiry_minutes * 60)
        self.response = store.response

        if (store.response.errors is None or len(store.response.errors) > 0):
            return Message(False, "There were errors")
        return Message(True, "")

    def __post(self, param_dictionary):
        """
        Perform Actual POST Request

        Args:
            param_dictionary: Type[dict], contains `function_name`, `token`(optional)

        Returns:
            requests HTTP Response Object
        """
        try:
            headers = {"Content-type": "application/json", "Accept": "text/plain"}
            result = requests.post(url=self.url, headers=headers, data=param_dictionary)


            return self.__result_to_message(result)
        except Exception as e:
            self.cost = 0
            return Message(False, str(e))

    def __get(self, param_dictionary):
        """
        Perform Actual GET Request

        Args:
            param_dictionary: Type[dict], contains `function_name`, `token`(optional)

        Returns:
            requests HTTP Response Object
        """
        result = requests.get(self.url, params=param_dictionary)
        return self.__result_to_message(result)



"""
Prepares TQ Request Instance with a configuration to hit 
Corresponding API endpoint for a request_Method.
They replalce the key:value pair dictionary for each API with functions and arguments.

Eg (GET method):
request_ip_return(): http://operations.treasuryquants.com/?function_name=ip_return

GitHub Description:
    Contains all the functions that build the request parameters.
"""


# TreasuryQuants.com Ltd.
# email: contact@treasuryquants.com
# Note: this software is provided "as-is" under the agreed terms of your account.
#       For more information see https://treasuryquants.com/terms-of-services/


class Request:
    def __init__(self, params, needs_token=True):
        """
        Initializes TQ Request Class Object

        Args:
            params: Type[dict]
            needs_token: Type[bool]
        """
        self.params = params
        self.needs_token = needs_token


class ParamBuilder:
    _function_name = 'function_name'

    def build(self, params, function_name, needs_token=True):
        """
        Args:
            params: Type[dict]
            function_name: Type[str], function-API-endpoint
            needs_token: Type[bool]

        Returns:
            Type[TQ Request]
        """
        params[self._function_name] = function_name
        return Request(params, needs_token)


#
# initialization to get our IP and at the same time check if we can have connection
#
def request_ip_return():
    """
    Returns:
        Type[Request] (TQ Request Class Object), contains attributes: Type[dict] `params` and `needs_token`
    """
    request_collection = {}
    params = ParamBuilder().build(request_collection, function_name='ip_return', needs_token=False)
    return params


#
# Account related function calls
#


#
# to ensure the status of the user account.
#
def request_account_status(email):
    """
    Returns:
        Type[Request] (TQ Request Class Object), contains attributes: Type[dict] `params` and `needs_token`
    """
    params = {"email": str(email),}
    return ParamBuilder().build(params, function_name='account_status', needs_token=True)


#
# This is the FIRST step of the two-step process for creating a new account.
# For the second step see request_account_reset.
#
def request_account_create(email,password="", ip='127.0.0.1', callback_url="",
                                                           is_test=False):
    params = {"email": str(email),"password": str(password), 'ip':str(ip), 'url':str(callback_url), 'is_test':str(is_test)}
    return ParamBuilder().build(params, function_name='account_create', needs_token=False)

#
# request for regeneration of the activation key, for either activating the account or resetting password
#
def request_account_send_activation_key(email, callback_url="", is_test=False):
    params = {"email": str(email),  'url': str(callback_url), 'is_test': str(is_test)}
    return ParamBuilder().build(params, function_name='account_send_activation_key', needs_token=False)


#
# request for activating an account using activation_key
#
def request_account_activate( activation_key, is_test=False):
    params = {  'activation_key': str(activation_key), 'is_test': str(is_test)}
    return ParamBuilder().build(params, function_name='account_activate', needs_token=False)



#
# request for changing password.
#
def request_account_password_change(email, password,new_password, is_test=False):
    params = {"email": str(email),  'password': str(password),  'new_password': str(new_password), 'is_test': str(is_test)}
    return ParamBuilder().build(params, function_name='account_password_change', needs_token=False)



#
# request for changing IP.
#
def request_account_ip_change(email, password,new_ip, is_test=False):
    params = {"email": str(email),  'password': str(password),  'new_ip': str(new_ip), 'is_test': str(is_test)}
    return ParamBuilder().build(params, function_name='account_ip_change', needs_token=False)

#
# request for password and IP change.
#
def request_account_password_ip_change(email, password,new_password,new_ip, is_test=False):
    params = {"email": str(email),'new_password':str(new_password),  'password': str(password),  'new_ip': str(new_ip), 'is_test': str(is_test)}
    return ParamBuilder().build(params, function_name='account_password_ip_change', needs_token=False)

#
# request for password reset
#
def request_account_password_reset( activation_key,new_password, is_test=False):
    params = {  'activation_key': str(activation_key),  'new_password': str(new_password), 'is_test': str(is_test)}
    return ParamBuilder().build(params, function_name='account_password_reset', needs_token=False)


def request_account_reset(activation_key, email, password, ip, is_test=False):
    params = {"email": email, 'activation_key': activation_key, 'password': password, 'ip': ip, 'is_test': str(is_test)}
    return ParamBuilder().build(params, function_name='account_reset', needs_token=False)


def request_account_activation_key_status(activation_key):
    params = { 'activation_key': activation_key}
    return ParamBuilder().build(params, function_name='account_activation_key_status', needs_token=False)

#
# request to get user profile
#
def request_account_profile(email, password):
    params = {"email": str(email),  'password': str(password)}
    return ParamBuilder().build(params, function_name='account_profile', needs_token=False)




#
# General functions
#


#
# requests for a new token. The expiry of the new token is inside the xml result for each and individual result.
#
def request_account_token_create(email):
    """
    Returns:
        Type[TQ Request], Request Instance configured to hit API /?function_name=account_token_create Endpoint.
    """
    params = {"email": email}
    return ParamBuilder().build(params, function_name='account_token_create', needs_token=False)


def request_function_describe(item_name=""):
    params = {}
    if len(item_name.lstrip()) > 0:
        params = {'element': item_name}
    return ParamBuilder().build(params, 'describe')


def request_function_show_available(item_name=""):
    params = {}
    if len(item_name.lstrip()) > 0:
        params = {'element': item_name}
    return ParamBuilder().build(params, 'show_available')

    # workspace_show_files and workspace_delete_file use the same API workspace


def request_function_workspace_show_files():
    params = {'list': 'all'}
    return ParamBuilder().build(params, 'workspace')

    # workspace_show_files and workspace_delete_file use the same API workspace


def request_function_workspace_delete_file(file_name):
    params = {'delete': file_name}
    return ParamBuilder().build(params, 'workspace')


def request_function_market_swap_rates(asof, currency):
    params = {'asof': asof, 'currency': currency}
    return ParamBuilder().build(params, 'market_swap_rates')

def request_function_formatted_grid_swap_rates(from_date,to_date, currency):
    params = {'from_date':from_date,'to_date': to_date, 'currency': currency}
    return ParamBuilder().build(params, 'formatted_grid_swap_rates')

def request_function_formatted_grid_fx(base_date,base_currency):
    params = {'base_date':base_date, 'base_currency':base_currency}
    return ParamBuilder().build(params, 'formatted_grid_fx')


def request_function_market_fx_rates(asof, to_date, base_currency):
    params = {'asof': asof, 'base_currency': base_currency, 'to_date': to_date}
    return ParamBuilder().build(params, 'market_fx_rates')


def request_function_price_vanilla_swap(
        asof
        , type
        , notional
        , trade_date
        , trade_maturity
        , index_id
        , discount_id
        , floating_leg_period
        , fixed_leg_period
        , floating_leg_daycount
        , fixed_leg_daycount
        , fixed_rate
        , is_payer
        , spread
        , business_day_rule
        , business_centres
        , spot_lag_days
        , save_as=""):
    params = {
        "asof": str(asof)
        , "type": type
        , "notional": str(notional)
        , "trade_date": str(trade_date)
        , "trade_maturity": str(trade_maturity)
        , "index_id": index_id
        , "discount_id": discount_id
        , "floating_leg_period": floating_leg_period
        , "fixed_leg_period": fixed_leg_period
        , "floating_leg_daycount": floating_leg_daycount
        , "fixed_leg_daycount": fixed_leg_daycount
        , "fixed_rate": str(fixed_rate)
        , "is_payer": str(is_payer).lower()
        , "spread": str(spread)
        , "business_day_rule": business_day_rule
        , "business_centres": business_centres
        , "spot_lag_days": str(spot_lag_days)}
    if len(save_as.lstrip()) > 0:
        params['save_as'] = save_as
    return ParamBuilder().build(params, 'price_vanilla_swap')


def request_function_price_fx_forward(
        asof
        , type
        , trade_date
        , trade_expiry
        , pay_amount
        , pay_currency
        , receive_amount
        , receive_currency
        , save_as=""):
    params = {
        "asof": str(asof)
        , "type": type.lower()
        , "trade_date": str(trade_date)
        , "trade_expiry": str(trade_expiry)
        , "pay_amount": str(pay_amount)
        , "pay_currency": pay_currency.lower()
        , "receive_amount": str(receive_amount)
        , "receive_currency": receive_currency.lower()}
    if len(save_as.lstrip()) > 0:
        params['save_as'] = save_as
    return ParamBuilder().build(params, 'price_fx_forward')


def request_function_price(asof, load_as):
    params = {'asof': asof, 'load_as': load_as}
    return ParamBuilder().build(params, 'price')


def request_function_risk_ladder(asof, load_as):
    params = {'asof': asof, 'load_as': load_as}
    return ParamBuilder().build(params, 'risk_ladder')


def request_function_pnl_predict(load_as, from_date, to_date):
    params = {'load_as': load_as, 'from_date': from_date, 'to_date': to_date}
    return ParamBuilder().build(params, 'pnl_predict')


def request_function_pnl_attribute(load_as, from_date, to_date):
    params = {'load_as': load_as, 'from_date': from_date, 'to_date': to_date}
    return ParamBuilder().build(params, 'pnl_attribute')






############################################### IMPORTANT ###############################################
#                          ANY CHANGE TO THIS FILE REQUIRES TESTING THE TARGET WEBSITES USING IT
#########################################################################################################




from . import TQRequests


def make_results(message, connection):
    if not message.is_OK:
        return [message.is_OK, connection.response.errors]
    else:
        return [message.is_OK, connection.response.results]


def connection_is_ok(connection):
    #
    # Check if we have connections
    #
    request_ip_return = TQRequests.request_ip_return()
    message = connection.send(request_ip_return)
    return make_results(message, connection)


def account_status(connection, user_email):
    #
    # check the status of the account
    #
    request_account_create = TQRequests.request_account_status(user_email)
    message = connection.send(request_account_create)
    return make_results(message, connection)


def account_create(connection, user_email, user_password, user_ip, callback_url, is_test):
    #
    # Creates a  account
    #
    request_account_create = TQRequests.request_account_create(user_email, user_password, user_ip, callback_url,
                                                               is_test)
    message = connection.send(request_account_create)
    return make_results(message, connection)


def account_activation_key_status(connection, activation_key):
    #
    # Activates a  (disabled) account
    #
    request_account_activation_key_status = TQRequests.request_account_activation_key_status(activation_key)
    message = connection.send(request_account_activation_key_status)


    return make_results(message, connection)


def account_activate(connection,  activation_key, is_test):
    #
    # Activates a  (disabled) account
    #
    request_account_activate = TQRequests.request_account_activate(activation_key, is_test)
    message = connection.send(request_account_activate)
    return make_results(message, connection)


def account_send_activation_key(connection, user_email, callback_url, is_test):
    #
    # sends (resends) activation key
    #
    request_account_send_activation_key = TQRequests.request_account_send_activation_key(user_email, callback_url,
                                                                                         is_test)
    message = connection.send(request_account_send_activation_key)
    return make_results(message, connection)



def account_password_change(connection,user_email, password, _password, is_test):
    #
    # Changes the existing password
    #
    request_account_password_change = TQRequests.request_account_password_change(user_email, password, _password,
                                                                                 is_test)
    message = connection.send(request_account_password_change)
    return make_results(message, connection)


def account_ip_change(connection,user_email, password, _ip, is_test):
    #
    # Changes the registered IP
    #
    request_account_ip_change = TQRequests.request_account_ip_change(user_email, password, _ip, is_test)
    message = connection.send(request_account_ip_change)
    return make_results(message, connection)


def account_password_reset(connection,  activation_key, _password, is_test):
    #
    # Changes/resets the password 
    #
    request_account_password_reset_dict = TQRequests.request_account_password_reset(activation_key,
                                                                                    _password,
                                                                                    is_test)
    message = connection.send(request_account_password_reset_dict)
    return make_results(message, connection)


def account_password_ip_change(connection, user_email, password, new_password, new_ip, is_test):
    #
    # Changes password and IP at the same time
    #
    request_account_password_ip_change = TQRequests.request_account_password_ip_change(user_email, password,
                                                                                       new_password, new_ip,is_test)
    message = connection.send(request_account_password_ip_change)
    return make_results(message, connection)


def account_profile(connection, user_email, password):
    #
    # Changes/resets the password
    #
    request_account_profile_dict = TQRequests.request_account_profile(user_email, password)
    message = connection.send(request_account_profile_dict)
    return make_results(message, connection)

def formatted_grid_swap_rates(connection, from_date, to_date,currency):
    #
    # Changes/resets the password
    #
    request_formatted_grid_swap_rates = TQRequests.request_function_formatted_grid_swap_rates(from_date,to_date,currency)
    message = connection.send(request_formatted_grid_swap_rates)
    return make_results(message, connection)

def formatted_grid_fx(connection, base_date, base_currency):
    #
    # Changes/resets the password
    #
    request_formatted_grid_fx = TQRequests.request_function_formatted_grid_fx(base_date,base_currency)
    message = connection.send(request_formatted_grid_fx)
    return make_results(message, connection)



import os
from .TQConnection import Connection, Message
from .TQConnection import TQRequests


def make_request( param, function_name):
    return TQRequests.ParamBuilder().build(param, function_name)


def delete_file(result_new_file_path):
    try:
        os.remove(result_new_file_path)
    except:
        pass
    return Message(True, "")


def write_test_file(file_path,section_tag, params, delimiter_char, comment_char):
    try:
        writefile = open(file_path, "w")
        for name, param in params:
            line = section_tag+":"+name+"\n"
            writefile.write(line)
            for key, value in param.items():
                line = key.lstrip()+ delimiter_char + value + "\n"
                writefile.write(line)
            writefile.write("\n")
        writefile.close()
        return Message(True, "")
    except Exception as e:
        return Message(False, str(e))


def read_test_file(file_path, section_tag, delimiter, comment):
    params = list()
    param = None
    try:
        line_cntr = 0
        inputfile = open(file_path, "r")
        for line in inputfile:
            # ignore blank lines
            if len(line.lstrip()) == 0:
                line_cntr += 1
                continue
            loc = line.rfind(comment)
            if loc == 0:
                line_cntr += 1
                continue

            #
            # a proper line to process
            #
            #has it got comment?
            if comment in line:
                line=line.split(comment)[0]
            #empty line on the ledt side?
            if line.rstrip().lstrip()=='':
                continue

            if section_tag in line:
                tag_name=""
                if ':' in line:
                    tag_name=line.split(':')[1].lstrip().rstrip().replace('\n','')
                param = dict()
                params.append((tag_name,param))
                continue

            if line.find(delimiter)==-1:
                line_cntr += 1
                return Message(False,
                               "Missing delimiter {} in file {} line:{}".format(delimiter, file_path,
                                                                                line_cntr)), params
            tokens = line.split(delimiter)
            tokens[0] = tokens[0].lstrip().rstrip()
            tokens[1] = tokens[1].lstrip().rstrip().replace('"', '')
            if len(tokens[0]) == 0:
                line_cntr += 1
                return Message(False,
                               "Missing delimiter key in key-value pair in file {} line:{}".format(file_path,
                                                                                                   line_cntr)), params
            if len(tokens[1]) == 0:
                line_cntr += 1
                return Message(False,
                               "Missing delimiter value in key-value pair in file {} line:{}".format(file_path,
                                                                                                     line_cntr)), params
            param[tokens[0]] = tokens[1]
            line_cntr += 1
        inputfile.close()
        return Message(True, ""), params
    except Exception as e:
        return Message(False, str(e)), params

def _process_diff(result_new_file_path,result_section_tag,result_news,delimiter_char,comment_char):
    Message = write_test_file(result_new_file_path, result_section_tag, result_news, delimiter_char,
                              comment_char)
    Message.content = "New results generated!"
    return Message


class Runner:
    def __init__(self, email,target_url):
        self.__param_factory = [
            'describe'
            ,'account_status'
            , 'account_password_change'
            , 'account_ip_change'
            , 'account_create'
            , 'account_send_activation_key'
            , 'account_activate'
            , 'account_profile'
            , 'account_activation_key_status'
            , 'ip_return'
            , 'market_fx_rates'
            , 'show_available'
            , 'workspace'
            , 'market_swap_rates'
            , 'pnl_attribute'
            , 'pnl_predict'
            , 'price'
            , 'price_fx_forward'
            , 'price_vanilla_swap'
            , 'risk_ladder'
            , 'formatted_grid_rates'
            , 'formatted_grid_fx'
        ]
        self.connection = Connection(email,target_url)

    def validate(self, params):
        if 'function_name' not in params:
            return Message(False, "There is no value for the key 'function_name'.")
        function_name = params['function_name']
        if function_name not in self.__param_factory:
            return Message(False, "function_name '" + function_name + "' is not recognised.")
        # param_builder = self.__param_factory[function_name]
        # Message = param_builder.validate(params)
        # if not Message.is_OK:
        #     return Message
        return Message(True, "")

    def send(self, params):
        function_name = params['function_name']
        #param_builder = self.__param_factory[function_name]
        request = make_request(params, function_name)

        Message = self.connection.send(request)
        return Message

    def get_response(self):
        return self.connection.response

    def get_all_test_file_names(self, root_folder, request_extension):
        file_names = list()
        for root, dirs, files in os.walk(root_folder):
            for filename in files:
                fname, fextension = os.path.splitext(filename)
                if fextension.lower() == "." + request_extension.lower():
                    file_names.append(fname)
        return file_names

    def run(self, root_folder, request_extension='request', result_extension='result',
            result_new_extension='result_new'):
        report = dict()
        test_file_names = self.get_all_test_file_names(root_folder, request_extension)
        for test_file_name in test_file_names:
            Message = self.execute_single_test(root_folder, test_file_name, request_extension, result_extension,
                                             result_new_extension)
            status_result = "OK"
            if len(Message.content) > 0:
                status_result = Message.content
            report[test_file_name] = status_result
            print ("{}:{}".format(test_file_name,status_result))
        return report

    def execute_single_test(self, root_folder, test_file_name, request_extension="request", result_extension="result", result_new_extension="result_new"):
        delimiter_char = '='
        comment_char = '#'
        test_section_tag = "[test]"
        result_section_tag = "[result]"

        volatile_list=['activation_key', 'balance', 'ip', 'last_login','ip_return', 'id']

        request_file_path = os.path.join(root_folder, test_file_name + "." + request_extension)
        result_file_path = os.path.join(root_folder, test_file_name + "." + result_extension)
        result_new_file_path = os.path.join(root_folder, test_file_name + "." + result_new_extension)

        Message, names_with_params = read_test_file(request_file_path,test_section_tag, delimiter_char, comment_char)

        result_news=list()
        if not Message.is_OK:
            return Message

        for name,param in names_with_params:
            Message = self.validate(param)
            if not Message.is_OK:
                return Message

            Message = self.send(param)
            return_values=dict()
            return_values= self.connection.response.results
            if not Message.is_OK:
                return_values = self.connection.response.errors

            return_values_clean=dict()
            for key, value in  return_values.items():
                return_values_clean[key]=value.rstrip().lstrip()

            result_news.append((name,return_values_clean))
            Message = delete_file(result_new_file_path)
            if not Message.is_OK:
                return Message



        Message, base_results = read_test_file(result_file_path,result_section_tag, delimiter_char, comment_char)

        if (len(base_results) == 0):
            Message = write_test_file(result_file_path,result_section_tag, result_news, delimiter_char, comment_char)
        else:
            if len(result_news) !=len(base_results):
                Message = _process_diff(result_new_file_path, result_section_tag, result_news, delimiter_char,
                                        comment_char)
            else:
                for i in range(0,len(result_news)):
                    base_name=base_results[i][0]
                    new_names=result_news[i][0]
                    if new_names!=base_name:
                        Message = _process_diff(result_new_file_path, result_section_tag, result_news, delimiter_char, comment_char)
                    else:
                        base_param=base_results[i][1]
                        new_param=result_news[i][1]
                        base_keys=base_param.keys()
                        new_keys=new_param.keys()
                        if new_keys!=base_keys:
                            Message = _process_diff(result_new_file_path, result_section_tag, result_news, delimiter_char,
                                                    comment_char)
                        else:
                            for base_key in base_keys:
                                if (base_key not in volatile_list) and new_param[base_key] != base_param[base_key]:
                                    Message = _process_diff(result_new_file_path, result_section_tag, result_news,
                                                            delimiter_char, comment_char)

        return Message


def run_test_all(folder, email,  target_url="http://operations.treasuryquants.com"):

    runner = Runner(email,  target_url)
    return runner.run(folder)


def run_test_single(root_folder, file_path,email, target_url="http://operations.treasuryquants.com"):

    runner = Runner(email, target_url)
    status_result = "OK"
    Message=runner.execute_single_test(root_folder,file_path)
    if len(Message.content) > 0:
        status_result = Message.content
    print(file_path, status_result)
    report={file_path:status_result}
    return report






import xml.etree.ElementTree as ET
import json

class ResponseClass:
    def __init__(self, focus='error', id='', cost=0, expiry_minutes=0, balance=0):
        self.focus = focus
        self.id = id
        self.cost = cost
        self.balance = balance
        self.expiry_minutes = expiry_minutes
        self.results = {}
        self.errors = {}
        self.logs = {}
        self.notes = []

    def fromXml(self, string_xml):
        """
        Parses GET/POST resposne XML-content &
        extract TQ Response-attributes (focus, id, cost, balance, expiry_minutes, results, errors) &
        Update TQ Response Instance with these values.
        """
        store_element = ET.fromstring(string_xml)
        response_element = store_element[0]
        self.cost = float(response_element.attrib['cost'])
        self.balance = float(response_element.attrib['balance'])
        self.focus = response_element.attrib['focus']
        self.id = response_element.attrib['id']
        self.expiry_minutes = float(response_element.attrib['expiry_minutes'])
        result_elements = store_element.findall('./Response/Results/Item')
        if result_elements is not None:
            for node in result_elements:
                tokens = node.text.split('=')
                self.results[tokens[0]] = tokens[1]

        error_elements = store_element.findall('./Response/Errors/Item')
        if error_elements is not None:
            for node in error_elements:
                tokens = node.text.split('=')
                self.errors[tokens[0]] = tokens[1]

        log_elements = store_element.findall('./Response/Logs/Item')
        if log_elements is not None:
            for node in log_elements:
                tokens = node.text.split('=')
                self.logs[tokens[0]] = tokens[1]

        error_elements = store_element.findall('./Response/Note/Item')
        if error_elements is not None:
            for node in error_elements:
                self.notes.append(node.text)
        return


    def toxml(self,store_element):
        response_element = ET.SubElement(store_element, 'Response')
        response_element.attrib['cost'] = "{:.2f}".format(float(self.cost))
        response_element.attrib['balance'] = "{:.2f}".format(float(self.balance))
        response_element.attrib['focus'] = self.focus
        response_element.attrib['id'] = self.id
        response_element.attrib['expiry_minutes'] = "{:.2f}".format(float(self.expiry_minutes))
        results_element = ET.SubElement(response_element, 'Results')
        for key, value in self.results.items():
            item = ET.SubElement(results_element, 'Item')
            item.text = key + "=" + value

        error_element = ET.SubElement(response_element, 'Errors')
        for key, value in self.errors.items():
            item = ET.SubElement(error_element, 'Item')
            item.text = key + "=" + value

        logs_element = ET.SubElement(response_element, 'Logs')
        for key, value in self.logs.items():
            item = ET.SubElement(logs_element, 'Item')
            item.text = key + "=" + value

        notes_element = ET.SubElement(response_element, 'Notes')
        for value in self.notes:
            item = ET.SubElement(notes_element, 'Item')
            item.text = value




    def fromJson(self, string_json):
        """
        Parses GET/POST resposne JSON-content &
        extract TQ Response-attributes (focus, id, cost, balance, expiry_minutes, results, errors) &
        Update TQ Response Instance with these values.
        """
        store_element=json.loads(string_json)
        response_element = store_element['Response']
        self.cost = float(response_element['cost'])
        self.balance = float(response_element['balance'])
        self.focus = response_element['focus']
        self.id = response_element['id']
        self.expiry_minutes = float(response_element['expiry_minutes'])

        result_elements = response_element['Results']
        if result_elements is not None:
            for key, value in result_elements.items():
                self.results[key] = value

        errors_elements = response_element['Errors']
        if errors_elements is not None:
            for key, value in errors_elements.items():
                self.errors[key] =value

        log_elements = response_element['Logs']
        if log_elements is not None:
            for key, value in log_elements.items():
                self.logs[key] = value

        notes_elements = response_element['Notes']
        if notes_elements is not None:
            for note in notes_elements:
                self.notes.append(note)
        return


    def toJson(self, store_node):
        store_node['Response'] =dict()
        response_node=store_node['Response']
        response_node['cost'] = "{:.2f}".format(float(self.cost))
        response_node['balance'] = "{:.2f}".format(float(self.balance))
        response_node['focus'] = self.focus
        response_node['id'] = self.id
        response_node['expiry_minutes'] = "{:.2f}".format(float(self.expiry_minutes))
        response_node['Results']=dict()
        results_node=response_node['Results']

        for key, value in self.results.items():
            results_node[key] = value

        response_node['Errors']=dict()
        errors_node=response_node['Errors']
        for key, value in self.errors.items():
            errors_node[key]=value

        response_node['Logs']=dict()
        logs_node=response_node['Logs']
        for key, value in self.logs.items():
            logs_node[key]= value

        response_node['Notes']=list()
        notes_node=response_node['Notes']

        for value in self.notes:
            notes_node.append(value)



class StoreClass:
    def __init__(self, client_id='', session_id='', id='', version='', note='', source_id='', response=None):
        self.client_id = client_id
        self.session_id = session_id
        self.id = id
        self.version = version
        self.note = note
        self.source_id = source_id
        self.response = response
        if self.response is None:
            self.response=ResponseClass()

    def toxml(self):
        store_element = ET.Element('Store')
        store_element.attrib["client_id"] = self.client_id
        store_element.attrib["session_id"] = self.session_id
        store_element.attrib["id"] = self.id
        store_element.attrib["version"] = self.version
        store_element.attrib["note"] = self.note
        store_element.attrib["source_id"] = self.source_id
        self.response.toxml(store_element)

        return ET.tostring(store_element, encoding='utf8', method='xml')

    def fromXml(self, string_xml):
        """
        Parses GET/POST resposne XML-content &
        extract Store-attributes (client_id, source_id, session_id, id, version, note) &
        triggers extraction of TQ Response Class attributes.
        """
        store_element = ET.fromstring(string_xml)
        self.client_id=store_element.attrib["client_id"]
        self.session_id=store_element.attrib["session_id"]
        self.id=store_element.attrib["id"]
        self.version=store_element.attrib["version"]
        self.note=store_element.attrib["note"]
        self.source_id=store_element.attrib["source_id"]
        self.response.fromXml(string_xml)


    def toJson(self):
        """
        Parses GET/POST resposne JSON-content &
        extract Store-attributes (client_id, source_id, session_id, id, version, note) &
        triggers extraction of TQ Response Class attributes.
        """
        store_node=dict()
        store_element = ET.Element('Store')
        store_node["client_id"] = self.client_id
        store_node["session_id"] = self.session_id
        store_node["id"] = self.id
        store_node["version"] = self.version
        store_node["note"] = self.note
        store_node["source_id"] = self.source_id
        self.response.toJson(store_node)
        return json.dumps(store_node)

    def fromJson(self, string_json):
        """
        Parses GET/POST resposne JSON-content &
        extract Store-attributes (client_id, source_id, session_id, id, version, note) &
        triggers extraction of TQ Response Class attributes.
        """
        store_element = json.loads(string_json)
        self.client_id=store_element["client_id"]
        self.session_id=store_element["session_id"]
        self.id=store_element["id"]
        self.version=store_element["version"]
        self.note=store_element["note"]
        self.source_id=store_element["source_id"]
        self.response.fromJson(string_json)



# input = "<?xml version='1.0' encoding='utf8'?><Store client_id='00000836' session_id='na' id='29521836882698' version='1.0' note='na' source_id='217.42.124.18'><Response cost='0.35' balance='999.65' focus='results' id='29521836882698' expiry_minutes='0.49'><Results><Item>Price=0.185732291648</Item><Item>Currency=eur</Item></Results><Errors /><Logs><Item>save_as=File was saved as test_ir_swap.</Item></Logs><Notes /></Response></Store>"
#
# store= StoreClass()
# store.fromXml(input)
# json_string1=store.toJson()
# store.fromJson(json_string1)
# json_string2=store.toJson()
# store.fromJson(json_string2)
# print(store.toxml())
# ## now implement the JSON
#




from setuptools import setup
setup(name='TQapis',
version='0.12.4',
description="Python package providing a simple interface to support TreasuryQuants.com's http protocols",
url='https://github.com/treasuryquants/TQPython',
author='Shahram Alavian',
author_email='contact@treasuryquants.com',
maintainer='Tanmay Jain',
maintainer_email='tanmayjain.cs@gmail.com',
license='MIT', # if you want you can use "LICENSE.txt"
packages=['TQapis'],
zip_safe=False,
long_description=open('README.MD').read(),
long_description_content_type='text/markdown',
keywords =["Treasury", "Quant", "TreasuryQuants", "Swap", "Valuations", "Quantitative Finance"],
install_requires=["requests"])
